@@ -94,6 +94,7 @@ pub trait ChainRpc {
     ///       "compact_target": "0x1e083126",
     ///       "dao": "0xb5a3e047474401001bc476b9ee573000c0c387962a38000000febffacf030000",
     ///       "epoch": "0x7080018000001",
+    ///       "extra_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///       "hash": "0xa5f5c85987a15de25661e5a214f2c1449cd803f071acc7999820f25246471f40",
     ///       "nonce": "0x0",
     ///       "number": "0x400",
@@ -101,7 +102,6 @@ pub trait ChainRpc {
     ///       "proposals_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///       "timestamp": "0x5cd2b117",
     ///       "transactions_root": "0xc47d5b78b3c4c4c853e2a32810818940d0ee403423bea9ec7b8e566d9595206c",
-    ///       "uncles_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///       "version": "0x0"
     ///     },
     ///     "proposals": [],
@@ -123,9 +123,9 @@ pub trait ChainRpc {
     ///           {
     ///             "capacity": "0x18e64b61cf",
     ///             "lock": {
-    ///               "args": "0x",
     ///               "code_hash": "0x28e83a1277d48add8e72fadaa9248559e1b632bab2bd60b27955ebc4c03800a5",
-    ///               "hash_type": "data"
+    ///               "hash_type": "data",
+    ///               "args": "0x"
     ///             },
     ///             "type": null
     ///           }
@@ -213,6 +213,7 @@ pub trait ChainRpc {
     ///       "compact_target": "0x1e083126",
     ///       "dao": "0xb5a3e047474401001bc476b9ee573000c0c387962a38000000febffacf030000",
     ///       "epoch": "0x7080018000001",
+    ///       "extra_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///       "hash": "0xa5f5c85987a15de25661e5a214f2c1449cd803f071acc7999820f25246471f40",
     ///       "nonce": "0x0",
     ///       "number": "0x400",
@@ -220,7 +221,6 @@ pub trait ChainRpc {
     ///       "proposals_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///       "timestamp": "0x5cd2b117",
     ///       "transactions_root": "0xc47d5b78b3c4c4c853e2a32810818940d0ee403423bea9ec7b8e566d9595206c",
-    ///       "uncles_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///       "version": "0x0"
     ///     },
     ///     "proposals": [],
@@ -242,9 +242,9 @@ pub trait ChainRpc {
     ///           {
     ///             "capacity": "0x18e64b61cf",
     ///             "lock": {
-    ///               "args": "0x",
     ///               "code_hash": "0x28e83a1277d48add8e72fadaa9248559e1b632bab2bd60b27955ebc4c03800a5",
-    ///               "hash_type": "data"
+    ///               "hash_type": "data",
+    ///               "args": "0x"
     ///             },
     ///             "type": null
     ///           }
@@ -328,6 +328,7 @@ pub trait ChainRpc {
     ///     "compact_target": "0x1e083126",
     ///     "dao": "0xb5a3e047474401001bc476b9ee573000c0c387962a38000000febffacf030000",
     ///     "epoch": "0x7080018000001",
+    ///     "extra_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///     "hash": "0xa5f5c85987a15de25661e5a214f2c1449cd803f071acc7999820f25246471f40",
     ///     "nonce": "0x0",
     ///     "number": "0x400",
@@ -335,7 +336,6 @@ pub trait ChainRpc {
     ///     "proposals_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///     "timestamp": "0x5cd2b117",
     ///     "transactions_root": "0xc47d5b78b3c4c4c853e2a32810818940d0ee403423bea9ec7b8e566d9595206c",
-    ///     "uncles_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///     "version": "0x0"
     ///   }
     /// }
@@ -409,6 +409,7 @@ pub trait ChainRpc {
     ///     "compact_target": "0x1e083126",
     ///     "dao": "0xb5a3e047474401001bc476b9ee573000c0c387962a38000000febffacf030000",
     ///     "epoch": "0x7080018000001",
+    ///     "extra_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///     "hash": "0xa5f5c85987a15de25661e5a214f2c1449cd803f071acc7999820f25246471f40",
     ///     "nonce": "0x0",
     ///     "number": "0x400",
@@ -416,7 +417,6 @@ pub trait ChainRpc {
     ///     "proposals_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///     "timestamp": "0x5cd2b117",
     ///     "transactions_root": "0xc47d5b78b3c4c4c853e2a32810818940d0ee403423bea9ec7b8e566d9595206c",
-    ///     "uncles_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///     "version": "0x0"
     ///   }
     /// }
@@ -450,6 +450,17 @@ pub trait ChainRpc {
     /// ## Params
     ///
     /// * `tx_hash` - Hash of a transaction
+    /// * `verbosity` - result format which allows 0, 1 and 2. (**Optional**, the defaults to 2.)
+    ///
+    /// ## Returns
+    ///
+    /// When verbosity is 0 (deprecated): this is reserved for compatibility, and will be removed in the following release.
+    /// It return null as the RPC response when the status is rejected or unknown, mimicking the original behaviors.
+    ///
+    /// When verbosity is 1: The RPC does not return the transaction content and the field transaction must be null.
+    ///
+    /// When verbosity is 2: if tx_status.status is pending, proposed, or committed,
+    /// the RPC returns the transaction content as field transaction, otherwise the field is null.
     ///
     /// ## Examples
     ///
@@ -500,9 +511,9 @@ pub trait ChainRpc {
     ///         {
     ///           "capacity": "0x2540be400",
     ///           "lock": {
-    ///             "args": "0x",
     ///             "code_hash": "0x28e83a1277d48add8e72fadaa9248559e1b632bab2bd60b27955ebc4c03800a5",
-    ///             "hash_type": "data"
+    ///             "hash_type": "data",
+    ///             "args": "0x"
     ///           },
     ///           "type": null
     ///         }
@@ -515,13 +526,18 @@ pub trait ChainRpc {
     ///     },
     ///     "tx_status": {
     ///       "block_hash": null,
-    ///       "status": "pending"
+    ///       "status": "pending",
+    ///       "reason": null
     ///     }
     ///   }
     /// }
     /// ```
     #[rpc(name = "get_transaction")]
-    fn get_transaction(&self, tx_hash: H256) -> Result<Option<TransactionWithStatus>>;
+    fn get_transaction(
+        &self,
+        tx_hash: H256,
+        verbosity: Option<Uint32>,
+    ) -> Result<Option<TransactionWithStatus>>;
 
     /// Returns the hash of a block in the [canonical chain](#canonical-chain) with the specified
     /// `block_number`.
@@ -606,6 +622,7 @@ pub trait ChainRpc {
     ///     "compact_target": "0x1e083126",
     ///     "dao": "0xb5a3e047474401001bc476b9ee573000c0c387962a38000000febffacf030000",
     ///     "epoch": "0x7080018000001",
+    ///     "extra_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///     "hash": "0xa5f5c85987a15de25661e5a214f2c1449cd803f071acc7999820f25246471f40",
     ///     "nonce": "0x0",
     ///     "number": "0x400",
@@ -613,7 +630,6 @@ pub trait ChainRpc {
     ///     "proposals_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///     "timestamp": "0x5cd2b117",
     ///     "transactions_root": "0xc47d5b78b3c4c4c853e2a32810818940d0ee403423bea9ec7b8e566d9595206c",
-    ///     "uncles_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///     "version": "0x0"
     ///   },
     ///   "id": 42
@@ -688,9 +704,9 @@ pub trait ChainRpc {
     ///       "output": {
     ///         "capacity": "0x802665800",
     ///         "lock": {
-    ///           "args": "0x",
     ///           "code_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
-    ///           "hash_type": "data"
+    ///           "hash_type": "data",
+    ///           "args": "0x"
     ///         },
     ///         "type": null
     ///       }
@@ -1014,6 +1030,7 @@ pub trait ChainRpc {
     ///       "compact_target": "0x1e083126",
     ///       "dao": "0xb5a3e047474401001bc476b9ee573000c0c387962a38000000febffacf030000",
     ///       "epoch": "0x7080018000001",
+    ///       "extra_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///       "hash": "0xdca341a42890536551f99357612cef7148ed471e3b6419d0844a4e400be6ee94",
     ///       "nonce": "0x0",
     ///       "number": "0x400",
@@ -1021,7 +1038,6 @@ pub trait ChainRpc {
     ///       "proposals_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///       "timestamp": "0x5cd2b118",
     ///       "transactions_root": "0xc47d5b78b3c4c4c853e2a32810818940d0ee403423bea9ec7b8e566d9595206c",
-    ///       "uncles_hash": "0x0000000000000000000000000000000000000000000000000000000000000000",
     ///       "version": "0x0"
     ///     },
     ///     "proposals": [],
@@ -1043,9 +1059,9 @@ pub trait ChainRpc {
     ///           {
     ///             "capacity": "0x18e64b61cf",
     ///             "lock": {
-    ///               "args": "0x",
     ///               "code_hash": "0x28e83a1277d48add8e72fadaa9248559e1b632bab2bd60b27955ebc4c03800a5",
-    ///               "hash_type": "data"
+    ///               "hash_type": "data",
+    ///               "args": "0x"
     ///             },
     ///             "type": null
     ///           }
@@ -1082,6 +1098,10 @@ pub trait ChainRpc {
 
     /// Return various consensus parameters.
     ///
+    /// ## Returns
+    ///
+    /// If any hardfork feature has `epoch=null`, it means the feature will never be activated.
+    ///
     /// ## Examples
     ///
     /// Request
@@ -1107,6 +1127,15 @@ pub trait ChainRpc {
     ///         "dao_type_hash": null,
     ///         "epoch_duration_target": "0x3840",
     ///         "genesis_hash": "0x7978ec7ce5b507cfb52e149e36b1a23f6062ed150503c85bbf825da3599095ed",
+    ///         "hardfork_features": [
+    ///             { "rfc": "0028", "epoch_number": null },
+    ///             { "rfc": "0029", "epoch_number": null },
+    ///             { "rfc": "0030", "epoch_number": null },
+    ///             { "rfc": "0031", "epoch_number": null },
+    ///             { "rfc": "0032", "epoch_number": null },
+    ///             { "rfc": "0036", "epoch_number": null },
+    ///             { "rfc": "0038", "epoch_number": null }
+    ///         ],
     ///         "id": "main",
     ///         "initial_primary_epoch_reward": "0x71afd498d000",
     ///         "max_block_bytes": "0x91c08",
@@ -1186,6 +1215,7 @@ pub(crate) struct ChainRpcImpl {
 
 const DEFAULT_BLOCK_VERBOSITY_LEVEL: u32 = 2;
 const DEFAULT_HEADER_VERBOSITY_LEVEL: u32 = 1;
+const DEFAULT_GET_TRANSACTION_VERBOSITY_LEVEL: u32 = 2;
 
 impl ChainRpc for ChainRpcImpl {
     fn get_block(
@@ -1316,35 +1346,34 @@ impl ChainRpc for ChainRpcImpl {
         })
     }
 
-    fn get_transaction(&self, tx_hash: H256) -> Result<Option<TransactionWithStatus>> {
+    fn get_transaction(
+        &self,
+        tx_hash: H256,
+        verbosity: Option<Uint32>,
+    ) -> Result<Option<TransactionWithStatus>> {
         let tx_hash = tx_hash.pack();
-        let id = packed::ProposalShortId::from_tx_hash(&tx_hash);
+        let verbosity = verbosity
+            .map(|v| v.value())
+            .unwrap_or(DEFAULT_GET_TRANSACTION_VERBOSITY_LEVEL);
 
-        let tx = {
-            let tx_pool = self.shared.tx_pool_controller();
-            let fetch_tx_for_rpc = tx_pool.fetch_tx_for_rpc(id);
-            if let Err(e) = fetch_tx_for_rpc {
-                error!("send fetch_tx_for_rpc request error {}", e);
-                return Err(RPCError::ckb_internal_error(e));
-            };
-
-            fetch_tx_for_rpc.unwrap().map(|(proposed, tx)| {
-                if proposed {
-                    TransactionWithStatus::with_proposed(tx)
-                } else {
-                    TransactionWithStatus::with_pending(tx)
-                }
-            })
-        };
-
-        Ok(tx.or_else(|| {
-            self.shared
-                .snapshot()
-                .get_transaction(&tx_hash)
-                .map(|(tx, block_hash)| {
-                    TransactionWithStatus::with_committed(tx, block_hash.unpack())
-                })
-        }))
+        if verbosity == 0 {
+            // legacy mode
+            //  When verbosity is 0 (deprecated): this is reserved for compatibility,
+            //  and will be removed in the following release.
+            //  It return null as the RPC response when the status is rejected or unknown,
+            //  mimicking the original behaviors.
+            self.get_transaction_verbosity0(tx_hash)
+        } else if verbosity == 1 {
+            // The RPC does not return the transaction content and the field transaction must be null.
+            self.get_transaction_verbosity1(tx_hash)
+        } else if verbosity == 2 {
+            // if tx_status.status is pending, proposed, or committed,
+            // the RPC returns the transaction content as field transaction,
+            // otherwise the field is null.
+            self.get_transaction_verbosity2(tx_hash)
+        } else {
+            Err(RPCError::invalid_params("invalid verbosity level"))
+        }
     }
 
     fn get_block_hash(&self, block_number: BlockNumber) -> Result<Option<H256>> {
@@ -1393,7 +1422,11 @@ impl ChainRpc for ChainRpcImpl {
     }
 
     fn get_live_cell(&self, out_point: OutPoint, with_data: bool) -> Result<CellWithStatus> {
-        let cell_status = self.shared.snapshot().cell(&out_point.into(), with_data);
+        let cell_status = self
+            .shared
+            .snapshot()
+            .as_ref()
+            .cell(&out_point.into(), with_data);
         Ok(cell_status.into())
     }
 
@@ -1633,3 +1666,88 @@ impl ChainRpc for ChainRpcImpl {
         Ok(Some(median_time.into()))
     }
 }
+
+impl ChainRpcImpl {
+    fn get_transaction_verbosity0(
+        &self,
+        tx_hash: packed::Byte32,
+    ) -> Result<Option<TransactionWithStatus>> {
+        if let Some((tx, block_hash)) = self.shared.snapshot().get_transaction(&tx_hash) {
+            return Ok(Some(TransactionWithStatus::with_committed(
+                Some(tx),
+                block_hash.unpack(),
+            )));
+        }
+
+        let tx_pool = self.shared.tx_pool_controller();
+        let fetch_tx_for_rpc = tx_pool.fetch_tx_for_rpc(tx_hash);
+        if let Err(e) = fetch_tx_for_rpc {
+            error!("send fetch_tx_for_rpc request error {}", e);
+            return Err(RPCError::ckb_internal_error(e));
+        };
+
+        let ret = fetch_tx_for_rpc.unwrap().map(|(proposed, tx)| {
+            if proposed {
+                TransactionWithStatus::with_proposed(Some(tx))
+            } else {
+                TransactionWithStatus::with_pending(Some(tx))
+            }
+        });
+
+        Ok(ret)
+    }
+
+    fn get_transaction_verbosity1(
+        &self,
+        tx_hash: packed::Byte32,
+    ) -> Result<Option<TransactionWithStatus>> {
+        if let Some(tx_info) = self.shared.snapshot().get_transaction_info(&tx_hash) {
+            return Ok(Some(TransactionWithStatus::with_committed(
+                None,
+                tx_info.block_hash.unpack(),
+            )));
+        }
+
+        let tx_pool = self.shared.tx_pool_controller();
+        let tx_status = tx_pool.get_tx_status(tx_hash);
+        if let Err(e) = tx_status {
+            error!("send get_tx_status request error {}", e);
+            return Err(RPCError::ckb_internal_error(e));
+        };
+        let tx_status = tx_status.unwrap();
+
+        if let Err(e) = tx_status {
+            error!("get_tx_status from db error {}", e);
+            return Err(RPCError::ckb_internal_error(e));
+        };
+        let tx_status = tx_status.unwrap();
+        Ok(Some(TransactionWithStatus::status_only(tx_status)))
+    }
+
+    fn get_transaction_verbosity2(
+        &self,
+        tx_hash: packed::Byte32,
+    ) -> Result<Option<TransactionWithStatus>> {
+        if let Some((tx, block_hash)) = self.shared.snapshot().get_transaction(&tx_hash) {
+            return Ok(Some(TransactionWithStatus::with_committed(
+                Some(tx),
+                block_hash.unpack(),
+            )));
+        }
+
+        let tx_pool = self.shared.tx_pool_controller();
+        let transaction_with_status = tx_pool.get_transaction_with_status(tx_hash);
+        if let Err(e) = transaction_with_status {
+            error!("send get_transaction_with_status request error {}", e);
+            return Err(RPCError::ckb_internal_error(e));
+        };
+        let transaction_with_status = transaction_with_status.unwrap();
+
+        if let Err(e) = transaction_with_status {
+            error!("get transaction_with_status from db error {}", e);
+            return Err(RPCError::ckb_internal_error(e));
+        };
+        let transaction_with_status = transaction_with_status.unwrap();
+        Ok(Some(transaction_with_status))
+    }
+}
