@@ -364,20 +364,18 @@ impl PoolRpc for PoolRpcImpl {
         let tx: core::TransactionView = tx.into_view();
 
         if let Err(e) = match outputs_validator {
-            Some(OutputsValidator::WellKnownScriptsOnly) | None => {
-                WellKnownScriptsOnlyValidator::new(
-                    self.shared.consensus(),
-                    &self.well_known_lock_scripts,
-                    &self.well_known_type_scripts,
-                )
-                .validate(&tx)
-            }
-            Some(OutputsValidator::Passthrough) => Ok(()),
+            None | Some(OutputsValidator::Passthrough) => Ok(()),
+            Some(OutputsValidator::WellKnownScriptsOnly) => WellKnownScriptsOnlyValidator::new(
+                self.shared.consensus(),
+                &self.well_known_lock_scripts,
+                &self.well_known_type_scripts,
+            )
+            .validate(&tx),
         } {
             return Err(RPCError::custom_with_data(
                 RPCError::PoolRejectedTransactionByOutputsValidator,
                 format!(
-                    "The transction is rejected by OutputsValidator set in params[1]: {}. \
+                    "The transaction is rejected by OutputsValidator set in params[1]: {}. \
                     Please check the related information in https://github.com/nervosnetwork/ckb/wiki/Transaction-%C2%BB-Default-Outputs-Validator",
                     outputs_validator.unwrap_or(OutputsValidator::WellKnownScriptsOnly).json_display()
                 ),
@@ -412,10 +410,7 @@ impl PoolRpc for PoolRpcImpl {
         let tx_hash = tx.hash();
         match submit_tx.unwrap() {
             Ok(_) => Ok(tx_hash.unpack()),
-            Err(e) => match RPCError::downcast_submit_transaction_reject(&e) {
-                Some(reject) => Err(RPCError::from_submit_transaction_reject(reject)),
-                None => Err(RPCError::from_ckb_error(e)),
-            },
+            Err(reject) => Err(RPCError::from_submit_transaction_reject(&reject)),
         }
     }
 
