@@ -2,15 +2,19 @@ use crate::error::RPCError;
 use ckb_app_config::BlockAssemblerConfig;
 use ckb_chain::chain::ChainController;
 use ckb_dao::DaoCalculator;
-use ckb_jsonrpc_types::{Block, BlockTemplate, Cycle, JsonBytes, Script, Transaction};
+use ckb_jsonrpc_types::{
+    AsEpochNumberWithFraction, Block, BlockTemplate, Byte32, JsonBytes, Script, Transaction,
+};
 use ckb_logger::error;
 use ckb_network::{NetworkController, SupportProtocols};
 use ckb_shared::{shared::Shared, Snapshot};
 use ckb_store::ChainStore;
 use ckb_types::{
     core::{
+        self,
         cell::{
-            resolve_transaction, OverlayCellProvider, ResolvedTransaction, TransactionsProvider,
+            resolve_transaction_with_options, OverlayCellProvider, ResolveOptions,
+            ResolvedTransaction, TransactionsProvider,
         },
         BlockView,
     },
@@ -40,11 +44,14 @@ pub trait IntegrationTestRpc {
         block_assembler_message: Option<JsonBytes>,
     ) -> Result<H256>;
 
-    #[rpc(name = "broadcast_transaction")]
-    fn broadcast_transaction(&self, transaction: Transaction, cycles: Cycle) -> Result<H256>;
+    #[rpc(name = "notify_transaction")]
+    fn notify_transaction(&self, transaction: Transaction) -> Result<H256>;
 
     #[rpc(name = "generate_block_with_template")]
     fn generate_block_with_template(&self, block_template: BlockTemplate) -> Result<H256>;
+
+    #[rpc(name = "calculate_dao_field")]
+    fn calculate_dao_field(&self, block_template: BlockTemplate) -> Result<Byte32>;
 }
 
 pub(crate) struct IntegrationTestRpcImpl {
@@ -117,17 +124,21 @@ impl IntegrationTestRpc for IntegrationTestRpcImpl {
         block_assembler_message: Option<JsonBytes>,
     ) -> Result<H256> {
         let tx_pool = self.shared.tx_pool_controller();
+        let snapshot = Arc::clone(&self.shared.snapshot());
         let block_assembler_config = block_assembler_script.map(|script| BlockAssemblerConfig {
             code_hash: script.code_hash,
             hash_type: script.hash_type,
             args: script.args,
             message: block_assembler_message.unwrap_or_default(),
+            use_binary_version_as_message_prefix: false,
+            binary_version: "TEST".to_string(),
         });
         let block_template = tx_pool
             .get_block_template_with_block_assembler_config(
                 None,
                 None,
                 None,
+                snapshot,
                 block_assembler_config,
             )
             .map_err(|err| RPCError::custom(RPCError::Invalid, err.to_string()))?
@@ -136,33 +147,28 @@ impl IntegrationTestRpc for IntegrationTestRpcImpl {
         self.process_and_announce_block(block_template.into())
     }
 
-    fn broadcast_transaction(&self, transaction: Transaction, cycles: Cycle) -> Result<H256> {
-        let tx: packed::Transaction = transaction.into();
-        let hash = tx.calc_tx_hash();
-        let relay_tx = packed::RelayTransaction::new_builder()
-            .cycles(cycles.value().pack())
-            .transaction(tx)
-            .build();
-        let relay_txs = packed::RelayTransactions::new_builder()
-            .transactions(vec![relay_tx].pack())
-            .build();
-        let message = packed::RelayMessage::new_builder().set(relay_txs).build();
-
-        if let Err(err) = self
-            .network_controller
-            .broadcast(SupportProtocols::Relay.protocol_id(), message.as_bytes())
-        {
-            error!("Broadcast transaction failed: {:?}", err);
-            Err(RPCError::custom_with_error(
-                RPCError::P2PFailedToBroadcast,
-                err,
-            ))
-        } else {
-            Ok(hash.unpack())
+    fn notify_transaction(&self, tx: Transaction) -> Result<H256> {
+        let tx: packed::Transaction = tx.into();
+        let tx: core::TransactionView = tx.into_view();
+        let tx_pool = self.shared.tx_pool_controller();
+        let tx_hash = tx.hash();
+        if let Err(e) = tx_pool.notify_txs(vec![tx]) {
+            error!("send notify_txs request error {}", e);
+            return Err(RPCError::ckb_internal_error(e));
         }
+        Ok(tx_hash.unpack())
     }
 
     fn generate_block_with_template(&self, block_template: BlockTemplate) -> Result<H256> {
+        let dao_field = self.calculate_dao_field(block_template.clone())?;
+
+        let mut update_dao_template = block_template;
+        update_dao_template.dao = dao_field;
+        let block = update_dao_template.into();
+        self.process_and_announce_block(block)
+    }
+
+    fn calculate_dao_field(&self, block_template: BlockTemplate) -> Result<Byte32> {
         let snapshot: &Snapshot = &self.shared.snapshot();
         let consensus = snapshot.consensus();
         let parent_header = snapshot
@@ -178,30 +184,37 @@ impl IntegrationTestRpc for IntegrationTestRpcImpl {
 
         let transactions_provider = TransactionsProvider::new(txs.as_slice().iter());
         let overlay_cell_provider = OverlayCellProvider::new(&transactions_provider, snapshot);
+        let resolve_opts = {
+            let epoch_number = block_template.epoch.epoch_number();
+            let hardfork_switch = consensus.hardfork_switch();
+            ResolveOptions::new().apply_current_features(hardfork_switch, epoch_number)
+        };
 
-        let rtxs = txs.iter().map(|tx| {
-            resolve_transaction(
-                tx.clone(),
-                &mut seen_inputs,
-                &overlay_cell_provider,
-                snapshot,
-                Some(2048)
-            ).map_err(|err| {
-                error!(
-                    "resolve transactions error when generating block with block template, error: {:?}",
-                    err
-                );
-                RPCError::invalid_params(err.to_string())
+        let rtxs = txs
+            .iter()
+            .map(|tx| {
+                resolve_transaction_with_options(
+                    tx.clone(),
+                    &mut seen_inputs,
+                    &overlay_cell_provider,
+                    snapshot,
+                    resolve_opts,
+                )
+                .map_err(|err| {
+                    error!(
+                        "resolve transactions error when generating block \
+                         with block template, error: {:?}",
+                        err
+                    );
+                    RPCError::invalid_params(err.to_string())
+                })
             })
-        }).collect::<Result<Vec<ResolvedTransaction>>>()?;
+            .collect::<Result<Vec<ResolvedTransaction>>>()?;
 
-        let mut update_dao_template = block_template;
-        update_dao_template.dao = DaoCalculator::new(consensus, &snapshot.as_data_provider())
+        Ok(DaoCalculator::new(consensus, &snapshot.as_data_provider())
             .dao_field(&rtxs, &parent_header)
             .expect("dao calculation should be OK")
-            .into();
-        let block = update_dao_template.into();
-        self.process_and_announce_block(block)
+            .into())
     }
 }
 
